/*
 * An opener for regions of a memory card.
 *
 * Copyright (C) 2021, 2022, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <mutex>

#include <fsserver/opener_resource.h>

#include "msc_region_operations.h"



/* Support for providing access to memory card regions. */

class MscRegionOpener : public OpenerResource
{
protected:
    MscRegionOperations *_ops;
    std::mutex _lock;

public:
    explicit MscRegionOpener(ResourceRegistry *registry, MscRegionOperations *ops)
    : OpenerResource(registry), _ops(ops)
    {
    }

    virtual ~MscRegionOpener();

    /* Convenience methods determining different object types. */

    virtual bool accessing_directory(flags_t flags, fileid_t fileid);

    virtual bool accessing_file(flags_t flags, fileid_t fileid);

    virtual bool directory_is_empty(fileid_t fileid);

    /* File opening methods. */

    virtual long get_fileid(const char *path, flags_t flags, fileid_t *fileid);

    virtual long make_accessor(flags_t flags, fileid_t fileid,
                               Accessor **accessor);

    virtual long make_directory_accessor(flags_t flags, fileid_t fileid,
                                         DirectoryAccessor **accessor);

    /* Filesystem object access and manipulation methods. */

    virtual long make_directory_object(const char *path, sys_mode_t mode);

    virtual long remove_object(fileid_t fileid);

    virtual long rename_object(const char *source, const char *target);

    virtual long stat_object(const char *path, void *base, offset_t size);

    virtual long unlink_object(fileid_t parent_fileid, fileid_t fileid);
};

// vim: tabstop=4 expandtab shiftwidth=4
